/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.generator;

import java.math.BigInteger;
import java.util.LinkedHashMap;
import java.util.Map;

import ru.ispras.fortress.data.DataTypeId;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.testbase.TestBaseQuery;

public final class Utils {
  private Utils() {}

  public static Object getParameter(final TestBaseQuery query, final String name) {
    InvariantChecks.checkNotNull(query);
    InvariantChecks.checkNotNull(name);
    return query.getParameters().get(name);
  }

  public static boolean hasParameter(final TestBaseQuery query, final String name) {
    InvariantChecks.checkNotNull(query);
    InvariantChecks.checkNotNull(name);
    return query.getParameters().containsKey(name);
  }

  public static int getParameterAsInt(
      final TestBaseQuery query,
      final String name) {
    final Object parameter = getParameter(query, name);
    InvariantChecks.checkNotNull(parameter, "No such parameter: " + name);

    if (parameter instanceof Number) {
      return ((Number) parameter).intValue();
    }

    if (parameter instanceof String) {
      return Integer.valueOf((String) parameter);
    }

    throw new NumberFormatException(String.format(
        "The %s parameter (%s) cannot be converted to int.",
        name, parameter.getClass().getSimpleName()));
  }

  public static BigInteger getParameterAsBigInteger(
      final TestBaseQuery query,
      final String name) {
    final Object parameter = Utils.getParameter(query, name);
    InvariantChecks.checkNotNull(parameter, "No such parameter: " + name);

    if (parameter instanceof BigInteger) {
      return (BigInteger) parameter;
    }

    if (parameter instanceof Long) {
      return BigInteger.valueOf(((Long) parameter).longValue());
    }

    if (parameter instanceof Integer) {
      return BigInteger.valueOf(((Integer) parameter).longValue());
    }

    if (parameter instanceof String) {
      return new BigInteger((String) parameter);
    }

    throw new NumberFormatException(String.format(
        "The %s parameter (%s) cannot be converted to BigInteger.",
        name, parameter.getClass().getSimpleName()));
  }

  public static boolean checkContextAttribute(
      final TestBaseQuery query,
      final String name,
      final String value) {
    InvariantChecks.checkNotNull(query);
    InvariantChecks.checkNotNull(name);
    InvariantChecks.checkNotNull(value);
    return value.equals(query.getContext().get(name));
  }

  public static Map<String, Node> extractUnknown(final TestBaseQuery query) {
    InvariantChecks.checkNotNull(query);

    final Map<String, Node> result = new LinkedHashMap<>();
    for (final Map.Entry<String, Node> e : query.getBindings().entrySet()) {
      final Node value = e.getValue();
      if (isUnknownVariable(value)) {
        result.put(e.getKey(), value);
      }
    }

    return result;
  }

  public static Map<String, Node> extractUnknownImms(final TestBaseQuery query) {
    InvariantChecks.checkNotNull(query);

    final Map<String, Node> result =  new LinkedHashMap<>();
    for (final Map.Entry<String, Node> e : query.getBindings().entrySet()) {
      final Node value = e.getValue();
      if (isUnknownVariable(value) &&
          value.getDataType().getTypeId() == DataTypeId.LOGIC_INTEGER) {
        result.put(e.getKey(), value);
      }
    }

    return result;
  }

  public static boolean isUnknownVariable(final Node node) {
    InvariantChecks.checkNotNull(node);

    if (node.getKind() != Node.Kind.VARIABLE) {
      return false;
    }

    if (((NodeVariable) node).getVariable().hasValue()) {
      return false;
    }

    return true;
  }
}
